package com.fr.plugin.third.party.jsdjjed;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.http.HttpEntity;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.client.config.RequestConfig;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpGet;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.conn.ssl.NoopHostnameVerifier;
import com.fr.third.org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import com.fr.third.org.apache.http.entity.StringEntity;
import com.fr.third.org.apache.http.entity.mime.MultipartEntityBuilder;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.impl.client.HttpClients;
import com.fr.third.org.apache.http.ssl.SSLContextBuilder;
import com.fr.third.org.apache.http.ssl.TrustStrategy;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.third.springframework.web.util.UriUtils;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.UUID;

public class Utils {
    public static String DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36";
    public static RequestConfig REQUEST_CONFIG = RequestConfig.custom()
            .setConnectionRequestTimeout(30000)
            .setSocketTimeout(30000) // 服务端相应超时
            .setConnectTimeout(30000) // 建立socket链接超时时间
            .build();

    public static CloseableHttpClient createSslHttpClient() {
        try {
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {

                @Override
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            }).build();
            HostnameVerifier hostnameVerifier = NoopHostnameVerifier.INSTANCE;
            SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, hostnameVerifier);
            return HttpClients.custom()
                    .setSSLSocketFactory(sslsf)
                    .build();
        } catch (Exception e) {
            LogKit.error(e.getMessage(), e);
        }
        return HttpClients.createDefault();
    }

    public static CloseableHttpClient createDefaultHttpClient() {
        CloseableHttpClient httpClient = HttpClients.custom()
                .build();
        return httpClient;
    }

    public static synchronized CloseableHttpClient createHttpClient(String url) {
        CloseableHttpClient httpClient = null;
        if (StringKit.isEmpty(url)) {
            httpClient = createDefaultHttpClient();
            return httpClient;
        }

        if (url.startsWith("https://")) {
            httpClient = createSslHttpClient();
            return httpClient;
        }
        httpClient = createDefaultHttpClient();
        return httpClient;
    }

    /**
     * 获取响应内容
     *
     * @param response
     * @return
     * @throws IOException
     */
    public static String getResponseBodyContent(CloseableHttpResponse response) throws IOException {
        if (response == null) {
            return "";
        }
        HttpEntity httpEntity = response.getEntity();
        if (httpEntity == null) {
            return "";
        }
        String content = EntityUtils.toString(httpEntity, "UTF-8");
        return content;
    }

    public static synchronized String createHttpGetContent(CloseableHttpClient httpClient, String url, String basicAuth, String contentType) throws IOException {
        if ((httpClient == null) || (StringKit.isEmpty(url))) {
            return "";
        }

        LogKit.info("http请求链接:" + url);
        HttpGet httpGet = new HttpGet(url);
        httpGet.addHeader("User-Agent", Utils.DEFAULT_USER_AGENT);
        if (StringKit.isNotEmpty(basicAuth)) {
            httpGet.addHeader("Authorization", basicAuth);
        }
        if (StringKit.isNotEmpty(contentType)) {
            httpGet.addHeader("Content-Type", contentType);
        }

        httpGet.setConfig(Utils.REQUEST_CONFIG);
        CloseableHttpResponse response = httpClient.execute(httpGet);
        int statusCode = response.getStatusLine().getStatusCode();
        String responseContent = getResponseBodyContent(response);
        LogKit.info("http响应内容:\n" + responseContent);
        if (statusCode != HttpStatus.SC_OK) {
            response.close();
            LogKit.error("http请求出错,http status:" + statusCode);
            return "";
        }
        response.close();
        if (StringKit.isEmpty(responseContent)) {
            LogKit.error("http请求出错,http响应内容为空");
            return "";
        }
        return responseContent;
    }

    public static String createHttpPostContent(CloseableHttpClient httpClient, String url, String bodyContent, String basicAuth, String contentType) throws IOException {
        return createHttpPostContent(httpClient, url, bodyContent, null, basicAuth, contentType);
    }

    public static String createHttpPostContent(CloseableHttpClient httpClient, String url, MultipartEntityBuilder multipartEntityBuilder, String basicAuth, String contentType) throws IOException {
        return createHttpPostContent(httpClient, url, "", multipartEntityBuilder, basicAuth, contentType);
    }

    public static synchronized String createHttpPostContent(CloseableHttpClient httpClient, String url, String bodyContent, MultipartEntityBuilder multipartEntityBuilder, String basicAuth, String contentType) throws IOException {
        if ((httpClient == null) || (StringKit.isEmpty(url))) {
            return "";
        }

        LogKit.info("http请求链接:" + url);
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("User-Agent", Utils.DEFAULT_USER_AGENT);
        httpPost.setConfig(Utils.REQUEST_CONFIG);
        if (StringKit.isNotEmpty(basicAuth)) {
            httpPost.addHeader("Authorization", basicAuth);
        }
        if (StringKit.isNotEmpty(contentType)) {
            httpPost.addHeader("Content-Type", contentType);
        }
        if (StringKit.isNotEmpty(bodyContent)) {
            LogKit.info("http请求内容:\n" + bodyContent);
            StringEntity bodyEntity = new StringEntity(bodyContent, "UTF-8");
            httpPost.setEntity(bodyEntity);
        }

        if (multipartEntityBuilder != null) {
            httpPost.setEntity(multipartEntityBuilder.build());
        }

        CloseableHttpResponse response = httpClient.execute(httpPost);
        String responseContent = getResponseBodyContent(response);
        LogKit.info("http响应内容:\n" + responseContent);
        int statusCode = response.getStatusLine().getStatusCode();
        if (statusCode != HttpStatus.SC_OK) {
            response.close();
            LogKit.error("http请求出错,http status:" + statusCode);
            return "";
        }
        response.close();
        if (StringKit.isEmpty(responseContent)) {
            LogKit.error("http请求出错,http响应内容为空");
            return "";
        }
        return responseContent;
    }


    public static String createHttpGetContent(CloseableHttpClient httpClient, String url, String contentType) throws IOException {
        return createHttpGetContent(httpClient, url, "", contentType);
    }

    public static String createHttpGetContentWithHttpClient(String url, String basicAuth, String contentType) throws IOException {
        CloseableHttpClient httpClient = Utils.createHttpClient(url);
        String content = createHttpGetContent(httpClient, url, basicAuth, contentType);
        httpClient.close();
        return content;
    }

    public static String createHttpGetContentWithHttpClient(String url) throws IOException {
        return createHttpGetContentWithHttpClient(url, "", "");
    }

    public static String createHttpPostContentWithHttpClient(String url, String bodyContent, String contentType) throws IOException {
        return createHttpPostContentWithHttpClient(url, bodyContent, "", contentType);
    }

    public static String createHttpPostContentWithHttpClient(String url, String bodyContent, String basicAuth, String contentType) throws IOException {
        CloseableHttpClient httpClient = Utils.createHttpClient(url);
        String content = createHttpPostContent(httpClient, url, bodyContent, basicAuth, contentType);
        httpClient.close();
        return content;
    }


    public static String createHttpPostContentWithHttpClient(String url, MultipartEntityBuilder multipartEntityBuilder, String contentType) throws IOException {
        return createHttpPostContentWithHttpClient(url, multipartEntityBuilder, "", contentType);
    }

    public static String createHttpPostContentWithHttpClient(String url, MultipartEntityBuilder multipartEntityBuilder, String basicAuth, String contentType) throws IOException {
        CloseableHttpClient httpClient = Utils.createHttpClient(url);
        String content = createHttpPostContent(httpClient, url, multipartEntityBuilder, basicAuth, contentType);
        httpClient.close();
        return content;
    }

    public static synchronized String createHttpPostContent(CloseableHttpClient httpClient, String url, String bodyContent, String contentType) throws IOException {
        return createHttpPostContent(httpClient, url, bodyContent, "", contentType);
    }


    public static String getRequestUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String url = req.getRequestURL().toString();
        return url;
    }

    /**
     * 获取完整请求链接
     *
     * @param req 请求
     * @return
     */
    public static String getFullRequestUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String url = req.getRequestURL().toString();
        String queryUrl = req.getQueryString();
        if ((queryUrl == null) || "null".equalsIgnoreCase(queryUrl)) {
            queryUrl = "";
        } else {
            queryUrl = "?" + queryUrl;
        }
        String fullUrl = url + queryUrl;
        return fullUrl;
    }


    public static boolean isDecisionLoginRequest(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String url = req.getRequestURL().toString();
        if (url.endsWith("/decision/login") || url.endsWith("/decision/login/")) {
            return true;
        }
        return false;
    }

    /**
     * 重定向
     *
     * @param res
     * @param url
     */
    public static void sendRedirect(HttpServletResponse res, String url) {
        if ((res == null) || (StringKit.isEmpty(url))) {
            return;
        }
        res.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }


    public static synchronized String getUuid() {
        String uuid = UUID.randomUUID().toString().replace("-", "");
        return uuid;
    }


    /**
     * 通过用户名删除用户,管理员用户无法删除
     *
     * @param username 用户名
     * @return
     * @throws Exception
     */
    public static int deleteUsersByUsername(String username) throws Exception {
        if (StringUtils.isEmpty(username)) {
            return 0;
        }
        User user = UserService.getInstance().getUserByUserName(username);
        if (user == null) {
            return 0;
        }
        String userId = user.getId();
        List<String> adminUserIds = UserService.getInstance().getAdminUserIdList();
        if ((adminUserIds != null) && (adminUserIds.size() >= 1) && (adminUserIds.contains(userId))) {
            return 0;
        }
        UserUpdateBean userUpdateBean = new UserUpdateBean();
        userUpdateBean.setRemoveUserIds(new String[]{userId});
        return UserService.getInstance().deleteUsers(userUpdateBean);
    }

    public static int deleteUsersByUsernames(List<String> usernames) throws Exception {
        if ((usernames == null) || (usernames.size() <= 0)) {
            return 0;
        }
        String username;
        int totalCount = 0, count;
        for (int i = 0, max = usernames.size() - 1; i <= max; i++) {
            username = usernames.get(i);
            count = deleteUsersByUsername(username);
            totalCount = totalCount + count;
        }
        return totalCount;
    }

    public static void editRealNameByUsername(String username, String realName) throws Exception {
        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(realName)) {
            return;
        }
        User user = UserService.getInstance().getUserByUserName(username);
        if (user == null) {
            return;
        }
        UserBean userBean = new UserBean(user.getEmail(), user.isEnable(), user.getMobile(), realName, user.getUserName(), user.getId());
        UserService.getInstance().editAccount(username, userBean);
    }

    public static JSONObject getSuccessResultJson() {
        return getResultJson("success", "");
    }

    public static JSONObject getFailuresResultJson() {
        return getResultJson("failure", "");
    }

    public static JSONObject getFailuresResultJson(String msg) {
        return getResultJson("failure", msg);
    }

    public static JSONObject getResultJson(String status, String msg) {
        JSONObject json = new JSONObject();
        json.put("status", status);
        json.put("msg", msg);
        return json;
    }

    public static String encodeUrlWithUtf8(String value) {
        if ((value == null) || (value.length() <= 0)) {
            return "";
        }
        /*String path = "/" + value;
        URI uri = new URI("http", "a", path, null);
        String tempValue = uri.toASCIIString();
        String encodedValue = tempValue.substring(9);*/
        String tempValue = "";
        try {
            tempValue = UriUtils.encodeQueryParam(value, "UTF-8");
            return tempValue;
        } catch (Exception e) {
            LogKit.error("Utils.encodeUrlWithUtf8:" + e.getMessage(), e);
            tempValue = value;
        }
        return tempValue;
    }
}
