package com.fr.plugin.third.party.jsdjjed.app.config;

import com.fanruan.api.query.QueryConditionKit;
import com.fanruan.api.query.RestrictionKit;
import com.fanruan.api.util.StringKit;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.third.org.apache.commons.collections4.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class AppConfigData {
    public static synchronized List<AppConfigEntity> queryAllAppConfigData() throws Exception {
        List<AppConfigEntity> entities = AppConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<AppConfigEntity>>() {
            @Override
            public List<AppConfigEntity> run(DAOContext daoContext) throws Exception {
                return daoContext.getDAO(AppConfigDao.class).find(QueryConditionKit.newQueryCondition());
            }
        });
        if (CollectionUtils.isEmpty(entities)) {
            return new ArrayList<>();
        }
        return entities;
    }

    public static synchronized AppConfigEntity queryAppConfigDataWithId(String id) throws Exception {
        AppConfigEntity entity = AppConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<AppConfigEntity>() {
            @Override
            public AppConfigEntity run(DAOContext daoContext) throws Exception {
                return daoContext.getDAO(AppConfigDao.class).getById(id);
            }
        });
        return entity;
    }

    public static synchronized List<AppConfigEntity> queryAppConfigDataWithConfigId(String configId) throws Exception {
        List<AppConfigEntity> entities = AppConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<AppConfigEntity>>() {
            @Override
            public List<AppConfigEntity> run(DAOContext daoContext) throws Exception {
                return daoContext.getDAO(AppConfigDao.class).find(QueryConditionKit.newQueryCondition().addRestriction(RestrictionKit.eq("configId", configId)));
            }
        });
        return entities;
    }

    public static synchronized AppConfigEntity queryAppConfigDataWithLogin() throws Exception {
        List<AppConfigEntity> entities = queryAppConfigDatasWithLogin();
        if (CollectionUtils.isEmpty(entities)) {
            return null;
        }
        return entities.get(0);
    }

    public static synchronized List<AppConfigEntity> queryAppConfigDatasWithLogin() throws Exception {
        List<AppConfigEntity> entities = AppConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<AppConfigEntity>>() {
            @Override
            public List<AppConfigEntity> run(DAOContext daoContext) throws Exception {
                return daoContext.getDAO(AppConfigDao.class).find(QueryConditionKit.newQueryCondition().addRestriction(RestrictionKit.eq("login", 1)));
            }
        });
        return entities;
    }


    public static AppConfigEntity queryAppConfigDataWithConfigIdAndOnlyOne(String id) throws Exception {
        List<AppConfigEntity> entities = queryAppConfigDataWithConfigId(id);
        if (CollectionUtils.isEmpty(entities)) {
            return null;
        }
        AppConfigEntity entity = entities.get(0);
        return entity;
    }

    public static boolean isAppConfigDataExistsWithConfigId(String configId) throws Exception {
        return isAppConfigDataExistsWithConfigId(configId,"");
    }


    public static boolean isAppConfigDataExistsWithConfigId(String configId, String id) throws Exception {
        AppConfigEntity entity = queryAppConfigDataWithConfigIdAndOnlyOne(configId);
        if (entity == null) {
            return false;
        }
        if (StringKit.isNotEmpty(id) && StringKit.equals(id, entity.getId())) {
            return false;
        }
        return true;
    }


    public static synchronized AppConfigEntity addAppConfigData(AppConfigEntity appConfigEntity) throws Exception {
        AppConfigEntity resultEntity = AppConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<AppConfigEntity>() {
            @Override
            public AppConfigEntity run(DAOContext daoContext) throws Exception {
                AppConfigEntity entity = appConfigEntity;
                entity.setId(UUID.randomUUID().toString());
                daoContext.getDAO(AppConfigDao.class).add(entity);
                return entity;
            }
        });
        return resultEntity;
    }

    public static synchronized void updateAppConfigData(AppConfigEntity appConfigEntity) throws Exception {
        AppConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<Boolean>() {
            @Override
            public Boolean run(DAOContext daoContext) throws Exception {
                daoContext.getDAO(AppConfigDao.class).update(appConfigEntity);
                return true;
            }
        });
    }

    public static synchronized AppConfigEntity deleteAppConfigDataWithId(String id) throws Exception {
        AppConfigEntity appConfigEntity = queryAppConfigDataWithId(id);
        if (appConfigEntity == null) {
            return null;
        }
        AppConfigEntity resultEntity = AppConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<AppConfigEntity>() {
            @Override
            public AppConfigEntity run(DAOContext daoContext) throws Exception {
                daoContext.getDAO(AppConfigDao.class).remove(appConfigEntity.getId());
                return appConfigEntity;
            }
        });
        return resultEntity;
    }

    public static synchronized AppConfigEntity deleteAppConfigDataWithConfigId(String configId) throws Exception {
        AppConfigEntity appConfigEntity = queryAppConfigDataWithConfigIdAndOnlyOne(configId);
        if (appConfigEntity == null) {
            return null;
        }
        return deleteAppConfigDataWithId(appConfigEntity.getId());
    }
}
