package com.fr.plugin.third.party.jsdjjed.feishu;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.third.party.jsdjjed.Utils;
import com.fr.plugin.third.party.jsdjjed.config.AppDataConfig;
import com.fr.third.org.apache.commons.collections4.CollectionUtils;
import com.fr.third.org.apache.commons.lang3.time.DateFormatUtils;
import com.fr.third.org.apache.http.entity.ContentType;
import com.fr.third.org.apache.http.entity.mime.HttpMultipartMode;
import com.fr.third.org.apache.http.entity.mime.MultipartEntityBuilder;
import com.fr.third.org.apache.http.entity.mime.content.FileBody;
import com.fr.third.org.apache.http.entity.mime.content.InputStreamBody;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.*;

public class FeishuApp {


    /**
     * 获取消息类型
     *
     * @param type
     * @return
     */
    public static String getMsgType(String type) {
        if ("post".equalsIgnoreCase(type)) {
            return "post";
        }
        return "text";
    }

    public static String getMsgTypeNotes(String type) {
        if ("post".equalsIgnoreCase(type)) {
            return "富文本";
        }
        return "文本";
    }



    /**
     * 获取发送类型
     *
     * @param type
     * @return
     */
    public static String getSendType(String type) {
        if ("file".equalsIgnoreCase(type)) {
            return "file";
        }
        if ("message_file".equalsIgnoreCase(type)) {
            return "message_file";
        }
        return "message";
    }

    public static String getSendTypeNotes(String type) {
        if ("file".equalsIgnoreCase(type)) {
            return "文件";
        }
        if ("message_file".equalsIgnoreCase(type)) {
            return "消息与文件";
        }
        return "消息";
    }

    public static boolean isSendMessage(String type) {
        if ("message".equalsIgnoreCase(type)) {
            return true;
        }
        if ("message_file".equalsIgnoreCase(type)) {
            return true;
        }
        return false;
    }


    public static boolean isSendFile(String type) {
        if ("file".equalsIgnoreCase(type)) {
            return true;
        }
        if ("message_file".equalsIgnoreCase(type)) {
            return true;
        }
        return false;
    }


    /**
     * 获取 app_access_token（企业自建应用）
     *
     * @param appId
     * @param appSecret
     * @return
     */
    public synchronized static String createAppAccessToken(String appId, String appSecret) throws IOException {
        if (StringKit.isEmpty(appId) || StringKit.isEmpty(appSecret)) {
            return "";
        }

        JSONObject bodyJson = new JSONObject();
        bodyJson.put("app_id", appId);
        bodyJson.put("app_secret", appSecret);

        String url = "https://open.feishu.cn/open-apis/auth/v3/app_access_token/internal";
        String configUrl = AppDataConfig.getInstance().getAccessTokenUrl();
        url = getRealValue(url, configUrl);
        url = getEscbUrl("accessTokenUrl", url);
        String content = Utils.createHttpPostContentWithHttpClient(url, bodyJson.toString(), "application/json; charset=utf-8");
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return "";
        }
        String token = contentJson.getString("app_access_token");
        return token;
    }


    /**
     * 获取登录用户身份
     *
     * @param codeValue
     * @param accessToken
     * @return
     * @throws IOException
     */
    public synchronized static FeishuLoginUserBean getLoginUserInfo(String codeValue, String accessToken) throws IOException {
        if (StringKit.isEmpty(codeValue) || StringKit.isEmpty(accessToken)) {
            return null;
        }

        JSONObject bodyJson = new JSONObject();
        bodyJson.put("grant_type", "authorization_code");
        bodyJson.put("code", codeValue);

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/authen/v1/access_token";
        String configUrl = AppDataConfig.getInstance().getCodeUserUrl();
        url = getRealValue(url, configUrl);
        String content = Utils.createHttpPostContentWithHttpClient(url, bodyJson.toString(), authorizationValue, "application/json; charset=utf-8");
        if (StringKit.isEmpty(content)) {
            return null;
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return null;
        }
        JSONObject dataJson = contentJson.getJSONObject("data");
        FeishuLoginUserBean userBean = new FeishuLoginUserBean();
        userBean.setUserId(dataJson.getString("user_id", ""));
        userBean.setName(dataJson.getString("name", ""));
        userBean.setEnName(dataJson.getString("en_name", ""));
        userBean.setMobile(dataJson.getString("mobile", ""));
        return userBean;
    }


    /**
     * 获取用户或机器人所在的群列表
     *
     * @param accessToken
     * @return
     * @throws IOException
     */
    public synchronized static List<FeishuChatGroupBean> getChatGroups(String accessToken) throws IOException {
        List<FeishuChatGroupBean> chatGroups = new ArrayList<>();
        if (StringKit.isEmpty(accessToken)) {
            return chatGroups;
        }

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/im/v1/chats";
        String configUrl = AppDataConfig.getInstance().getChatGroupsUrl();
        url = getRealValue(url, configUrl) + "?page_size=100";
        url = getEscbUrl("chatGroupsUrl", url) + "&page_size=100";
        String content = Utils.createHttpGetContentWithHttpClient(url, authorizationValue, "application/json; charset=utf-8");
        if (StringKit.isEmpty(content)) {
            return chatGroups;
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return chatGroups;
        }
        JSONObject dataJson = contentJson.getJSONObject("data");
        if (!contentJson.containsKey("data")) {
            return chatGroups;
        }
        if (dataJson == null) {
            return chatGroups;
        }

        if ((!dataJson.containsKey("items")) && (!dataJson.containsKey("groups"))) {
            return chatGroups;
        }

        JSONArray itemJsons = null;
        if (dataJson.containsKey("items")) {
            itemJsons = dataJson.getJSONArray("items");
        }
        if (dataJson.containsKey("groups")) {
            itemJsons = dataJson.getJSONArray("groups");
        }

        if ((dataJson == null) || (itemJsons.size() <= 0)) {
            return chatGroups;
        }

        JSONObject itemJson;
        FeishuChatGroupBean chatGroupBean;
        for (int i = 0, max = itemJsons.size() - 1; i <= max; i++) {
            itemJson = itemJsons.getJSONObject(i);
            chatGroupBean = new FeishuChatGroupBean();
            chatGroupBean.setChatId(itemJson.getString("chat_id", ""));
            chatGroupBean.setName(itemJson.getString("name", ""));
            chatGroupBean.setDescription(itemJson.getString("description", ""));
            chatGroupBean.setTenantKey(itemJson.getString("tenant_key", ""));
            chatGroups.add(chatGroupBean);
        }
        return chatGroups;
    }


    /**
     * 使用手机号或邮箱获取用户 ID
     *
     * @param emailSet
     * @param mobileSet
     * @param accessToken
     * @return
     * @throws IOException
     */
    public synchronized static List<String> getFeishuUserId(Set<String> emailSet, Set<String> mobileSet, String accessToken) throws IOException {
        List<String> feishuUserIds = new ArrayList<>();
        if (StringKit.isEmpty(accessToken)) {
            return feishuUserIds;
        }

        if (CollectionUtils.isEmpty(emailSet) && CollectionUtils.isEmpty(mobileSet)) {
            return feishuUserIds;
        }
        String emailsContent = getUrlParams("emails", emailSet);
        String mobilesContent = getUrlParams("mobiles", mobileSet);

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/user/v1/batch_get_id";
        String configUrl = AppDataConfig.getInstance().getUsersUrl();
        url = getRealValue(url, configUrl);
        url = getEscbUrl("usersUrl", url);
        int flag = 0;
        if (StringKit.isNotEmpty(emailsContent)) {
            if (url.indexOf("?") > 0) {
                url = url + "&" + emailsContent;
            } else {
                url = url + "?" + emailsContent;
            }
            flag++;
        }

        if (StringKit.isNotEmpty(mobilesContent)) {
            if (url.indexOf("?") > 0) {
                url = url + "&" + mobilesContent;
            } else {
                url = url + "?" + mobilesContent;
            }
            flag++;
        }
        if (flag == 0) {
            return feishuUserIds;
        }
        String content = Utils.createHttpGetContentWithHttpClient(url, authorizationValue, "application/json; charset=utf-8");
        if (StringKit.isEmpty(content)) {
            return feishuUserIds;
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return feishuUserIds;
        }

        JSONObject dataJson = contentJson.getJSONObject("data");
        if (dataJson == null) {
            return feishuUserIds;
        }

        Set<String> userIdSet = new HashSet<>();
        if (dataJson.containsKey("email_users")) {
            JSONObject emailUsersJson = dataJson.getJSONObject("email_users");
            userIdSet.addAll(getUserIdsByJson(emailUsersJson));
        }

        if (dataJson.containsKey("mobile_users")) {
            JSONObject mobileUsersJson = dataJson.getJSONObject("mobile_users");
            userIdSet.addAll(getUserIdsByJson(mobileUsersJson));
        }

        feishuUserIds.addAll(userIdSet);
        return feishuUserIds;
    }

    /**
     * 上传图片
     *
     * @param imageInputStream
     * @param accessToken
     * @return
     * @throws IOException
     */
    public synchronized static JSONObject uploadImage(InputStream imageInputStream, String accessToken) throws IOException {
        JSONObject fileJson = new JSONObject();
        if (StringKit.isEmpty(accessToken) || (imageInputStream == null)) {
            return fileJson;
        }
        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/open-apis/im/v1/images";
        String configUrl = AppDataConfig.getInstance().getUploadImageUrl();
        url = getRealValue(url, configUrl);
        url = getEscbUrl("uploadImageUrl", url);
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create();
        multipartEntityBuilder.addTextBody("image_type", "message");
        multipartEntityBuilder.addPart("image", new InputStreamBody(imageInputStream, "image.png"));
        String content = Utils.createHttpPostContentWithHttpClient(url, multipartEntityBuilder, authorizationValue, "");
        if (StringKit.isEmpty(content)) {
            return fileJson;
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return fileJson;
        }
        JSONObject dataJson = contentJson.getJSONObject("data");
        if (dataJson == null) {
            return fileJson;
        }
        return dataJson;
    }

    /**
     * 上传文件
     *
     * @param fileType
     * @param fileName
     * @param fileInputStream
     * @param accessToken
     * @return
     * @throws IOException
     */
    public synchronized static JSONObject uploadFile(String fileType, String fileName, InputStream fileInputStream, String accessToken) throws IOException {
        JSONObject fileJson = new JSONObject();
        if (StringKit.isEmpty(accessToken) || StringKit.isEmpty(fileName) || (fileInputStream == null)) {
            return fileJson;
        }
        String fileTypeValue = StringKit.trim(fileType);
        if (StringKit.isEmpty(fileTypeValue)) {
            fileTypeValue = "stream";
        } else {
            fileTypeValue = fileTypeValue.toLowerCase();
            String[] defaultFileTypes = {"opus", "mp4", "pdf", "doc", "xls", "ppt", "stream"};
            int flag = 0;
            for (int i = 0, max = defaultFileTypes.length - 1; i <= max; i++) {
                if (StringKit.equals(fileTypeValue, defaultFileTypes[i])) {
                    flag++;
                    break;
                }
            }
            if (flag == 0) {
                fileTypeValue = "stream";
            }
        }

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/open-apis/im/v1/files";
        String configUrl = AppDataConfig.getInstance().getUploadFileUrl();
        url = getRealValue(url, configUrl);
        url = getEscbUrl("uploadFileUrl", url);
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create();
        multipartEntityBuilder.addTextBody("file_type", fileTypeValue,ContentType.create("text/plain",Charset.forName("UTF-8")));
        multipartEntityBuilder.addTextBody("file_name", fileName,ContentType.create("text/plain",Charset.forName("UTF-8")));
        multipartEntityBuilder.addBinaryBody("file", fileInputStream);
        String content = Utils.createHttpPostContentWithHttpClient(url, multipartEntityBuilder, authorizationValue, "");
        if (StringKit.isEmpty(content)) {
            return fileJson;
        }
        JSONObject contentJson = new JSONObject(content);
        int code = contentJson.getInt("code");
        if (code != 0) {
            return fileJson;
        }
        JSONObject dataJson = contentJson.getJSONObject("data");
        if (dataJson == null) {
            return fileJson;
        }
        return dataJson;
    }

    public static boolean isValidFileJson(JSONObject fileJson) {
        if (fileJson == null) {
            return false;
        }

        if (fileJson.containsKey("file_key")) {
            return true;
        }
        return false;
    }

    public static boolean isValidImageJson(JSONObject fileJson) {
        if (fileJson == null) {
            return false;
        }

        if (fileJson.containsKey("image_key")) {
            return true;
        }
        return false;
    }

    /**
     * 发送消息
     *
     * @param receiveIdType
     * @param receiveId
     * @param msgContent
     * @param msgType
     * @param accessToken
     * @throws IOException
     */
    public synchronized static void sendMessage(String receiveIdType, String receiveId, String msgContent, String msgType, String accessToken) throws IOException {
        if (StringKit.isEmpty(receiveIdType) || StringKit.isEmpty(receiveId) || StringKit.isEmpty(msgContent) || StringKit.isEmpty(msgType) || StringKit.isEmpty(accessToken)) {
            return;
        }

        JSONObject bodyJson = new JSONObject();
        //bodyJson.put("receive_id_type", receiveIdType);
        bodyJson.put("receive_id", receiveId);
        bodyJson.put("content", msgContent);
        bodyJson.put("msg_type", msgType);

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/im/v1/messages";
        String configUrl = AppDataConfig.getInstance().getSendMessageUrl();
        url = getRealValue(url, configUrl) + "?receive_id_type=" + receiveIdType;
        url = getEscbUrl("sendMessageUrl", url) + "&receive_id_type=" + receiveIdType;
        String content = Utils.createHttpPostContentWithHttpClient(url, bodyJson.toString(), authorizationValue, "application/json; charset=utf-8");
    }

    /**
     * 发送文本消息
     *
     * @param receiveIdType
     * @param receiveId
     * @param textContent
     * @param accessToken
     * @throws IOException
     */
    public static void sendMessageWithText(String receiveIdType, String receiveId, String textContent, String accessToken) throws IOException {
        JSONObject textJson = new JSONObject();
        textJson.put("text", textContent);
        sendMessage(receiveIdType, receiveId, textJson.toString(), "text", accessToken);
    }

    /**
     * 发送群文本消息
     *
     * @param chatId
     * @param textContent
     * @param accessToken
     * @throws IOException
     */
    public static void sendGroupMessageWithText(String chatId, String textContent, String accessToken) throws IOException {
        sendMessageWithText("chat_id", chatId, textContent, accessToken);
    }

    public static void sendMessageWithImage(String receiveIdType, String receiveId, String imageContent, String accessToken) throws IOException {
        sendMessage(receiveIdType, receiveId, imageContent, "image", accessToken);
    }

    public static void sendGroupMessageWithImage(String chatId, String imageContent, String accessToken) throws IOException {
        sendMessageWithImage("chat_id", chatId, imageContent, accessToken);
    }

    public static void sendMessageWithFile(String receiveIdType, String receiveId, String fileContent, String accessToken) throws IOException {
        sendMessage(receiveIdType, receiveId, fileContent, "file", accessToken);
    }

    public static void sendMessageWithFileByUserId(String receiveId, String fileContent, String accessToken) throws IOException {
        sendMessage("user_id", receiveId, fileContent, "file", accessToken);
    }

    public static void sendGroupMessageWithFile(String chatId, String fileContent, String accessToken) throws IOException {
        sendMessageWithFile("chat_id", chatId, fileContent, accessToken);
    }


    public synchronized static void sendBatchMessage(List<String> userIds, String msgContent, String msgType, String accessToken) throws IOException {
        if (CollectionUtils.isEmpty(userIds) || StringKit.isEmpty(msgContent) || StringKit.isEmpty(msgType) || StringKit.isEmpty(accessToken)) {
            return;
        }

        JSONArray userIdJsons = new JSONArray();
        for (int i = 0, max = userIds.size() - 1; i <= max; i++) {
            userIdJsons.add(userIds.get(i));
        }
        JSONObject bodyJson = new JSONObject();
        bodyJson.put("content", new JSONObject(msgContent));
        bodyJson.put("msg_type", msgType);
        bodyJson.put("user_ids", userIdJsons);

        String authorizationValue = "Bearer " + accessToken;
        String url = "https://open.feishu.cn/open-apis/message/v4/batch_send";
        String configUrl = AppDataConfig.getInstance().getSendBatchMessageUrl();
        url = getRealValue(url, configUrl);
        url = getEscbUrl("sendBatchMessageUrl", url);
        String content = Utils.createHttpPostContentWithHttpClient(url, bodyJson.toString(), authorizationValue, "application/json; charset=utf-8");
    }


    private static String getEscbUrl(String type, String originalUrl) {
        boolean escbOption = false;
        String escbCode = "";
        String escbVersion = "";

        if ("accessTokenUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isAccessTokenUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getAccessTokenUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getAccessTokenUrlEscbVersion();
        } else if ("codeUserUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isCodeUserUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getCodeUserUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getCodeUserUrlEscbVersion();
        } else if ("chatGroupsUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isChatGroupsUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getChatGroupsUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getChatGroupsUrlEscbVersion();
        } else if ("usersUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isUsersUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getUsersUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getUsersUrlEscbVersion();
        } else if ("sendMessageUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isSendMessageUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getSendMessageUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getSendMessageUrlEscbVersion();
        } else if ("sendBatchMessageUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isSendBatchMessageUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getSendBatchMessageUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getSendBatchMessageUrlEscbVersion();
        } else if ("uploadImageUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isUploadImageUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getUploadImageUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getUploadImageUrlEscbVersion();
        } else if ("uploadFileUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isUploadFileUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getUploadFileUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getUploadFileUrlEscbVersion();
        } else if ("authorizeUrl".equals(type)) {
            escbOption = AppDataConfig.getInstance().isAuthorizeUrlEscbOption();
            escbCode = AppDataConfig.getInstance().getAuthorizeUrlEscbCode();
            escbVersion = AppDataConfig.getInstance().getAuthorizeUrlEscbVersion();
        } else {
            return originalUrl;
        }

        if ((!escbOption) || StringKit.isEmpty(escbCode)) {
            return originalUrl;
        }

        if (StringKit.isEmpty(escbVersion)) {
            escbVersion = "1.0";
        }

        String ssdpValue = "Api_ID=" + escbCode
                + "&Api_Version=" + escbVersion
                + "&App_Sub_ID=" + AppDataConfig.getInstance().getEscbAppCode()
                + "&App_Token=" + AppDataConfig.getInstance().getEscbAppToken()
                + "&Partner_ID=" + AppDataConfig.getInstance().getEscbOrgCode()
                + "&Sign=NO_SIGN"
                + "&Sys_ID=" + AppDataConfig.getInstance().getEscbSysCode()
                + "&Time_Stamp=" + DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss:SSS")
                + "&User_Token=";
        LogKit.info("飞书集成," + type + ",ssdp:" + ssdpValue);

        Base64.Encoder encoder = Base64.getEncoder();
        String url = AppDataConfig.getInstance().getEscbUrl() + "?ssdp=" + new String(encoder.encode(ssdpValue.getBytes()));
        LogKit.info("飞书集成,ssdp url:" + url);
        return url;
    }


    public static void sendBatchMessageWithText(List<String> userIds, String textContent, String accessToken) throws IOException {
        JSONObject textJson = new JSONObject();
        textJson.put("text", textContent);
        sendBatchMessage(userIds, textJson.toString(), "text", accessToken);
    }

    public static void sendBatchMessageWithRichText(List<String> userIds, String textContent, String accessToken) throws IOException {
        JSONObject textJson = new JSONObject();
        textJson.put("post", new JSONObject(textContent));
        sendBatchMessage(userIds, textJson.toString(), "post", accessToken);
    }


    public static void sendBatchMessageWithImage(List<String> userIds, String imageContent, String accessToken) throws IOException {
        sendBatchMessage(userIds, imageContent, "image", accessToken);
    }

    public static void sendBatchMessageWithFile(List<String> userIds, String fileContent, String accessToken) throws IOException {
        //sendBatchMessage(userIds, fileContent, "file", accessToken);
        //批量发送不支持发送文件
        if (CollectionUtils.isEmpty(userIds)) {
            return;
        }
        for (int i = 0, max = userIds.size() - 1; i <= max; i++) {
            sendMessageWithFileByUserId(userIds.get(i), fileContent, accessToken);
        }
    }


    private static List<String> getUserIdsByJson(JSONObject dataJson) {
        List<String> userIds = new ArrayList<>();
        if (dataJson == null) {
            return userIds;
        }

        Set<String> fieldNames = dataJson.fieldNames();
        if (CollectionUtils.isEmpty(fieldNames)) {
            return userIds;
        }

        JSONArray tempJsons;
        JSONObject tempJson;
        for (String fieldName : fieldNames) {
            tempJsons = dataJson.getJSONArray(fieldName);
            if ((tempJsons == null) || (tempJsons.size() <= 0)) {
                continue;
            }
            for (int i = 0, max = tempJsons.size() - 1; i <= max; i++) {
                tempJson = tempJsons.getJSONObject(i);
                userIds.add(tempJson.getString("user_id"));
            }
        }
        return userIds;
    }


    private static String getUrlParams(String paramName, Set<String> valueSet) {
        if (StringKit.isEmpty(paramName) || CollectionUtils.isEmpty(valueSet)) {
            return "";
        }
        List<String> values = new ArrayList<>();
        values.addAll(valueSet);
        String paramsContent = "";
        String value;
        int count = 0;
        for (int i = 0, max = values.size() - 1; i <= max; i++) {
            if (count >= 1) {
                paramsContent = paramsContent + "&";
            }
            value = values.get(i);
            if (StringKit.isEmpty(value)) {
                continue;
            }
            paramsContent = paramsContent + paramName + "=" + value;
            count++;
        }
        return paramsContent;
    }


    private static String getRealValue(String value, String value1) {
        String tempValue = StringKit.trim(value1);
        if (StringKit.isEmpty(tempValue)) {
            return value;
        }
        return tempValue;
    }
}
