package com.fr.plugin.third.party.jsdjjed.http;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.query.QueryConditionKit;
import com.fanruan.api.query.RestrictionKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.ComparatorUtils;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.third.party.jsdjjed.Utils;
import com.fr.plugin.third.party.jsdjjed.app.config.AppConfigData;
import com.fr.plugin.third.party.jsdjjed.app.config.AppConfigEntity;
import com.fr.plugin.third.party.jsdjjed.config.AppDataConfig;
import com.fr.plugin.third.party.jsdjjed.feishu.FeishuApp;
import com.fr.plugin.third.party.jsdjjed.feishu.FeishuLoginUserBean;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;


public class SessionGlobalRequestFilterProvider extends AbstractGlobalRequestFilterProvider {
    private static String DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36";

    @Override
    public String filterName() {
        return "ttt.com.fr.plugin.third.party.jsdjjed";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/decision", "/decision/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {

            String fullUrl = Utils.getFullRequestUrl(req);
            String method = req.getMethod();
            LogKit.info("飞书登录集成,记录访问地址：" + method + " " + fullUrl);
            if (!"GET".equalsIgnoreCase(method)) {
                filterChain.doFilter(req, res);
                return;
            }

            String reqUrl = Utils.getRequestUrl(req);

            if (Utils.isDecisionLoginRequest(req)) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/remote/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("terminal=H5") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("__device__=") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/weixin/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/dingtalk/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            String state;
            if (isAllowLoginWithParameter(req)) {
                reqUrl = getRequestUrl(req);
                state = UUID.randomUUID().toString();
                AppDataConfig.getInstance().addMapUrl(state, reqUrl);
                String tempUrl = getFullAuthorizeUrl(state);
                LogKit.info("飞书登录集成,请求认证地址：" + tempUrl);
                sendRedirect(res, tempUrl);
                return;
            }

            if (isAllowLoginWithLightApp(req)) {
                reqUrl = getRequestUrl(req);
                state = UUID.randomUUID().toString();
                AppDataConfig.getInstance().addMapUrl(state, reqUrl);
                LogKit.info("飞书登录集成,报表轻应用跳转");
                AppConfigEntity appConfigEntity = AppConfigData.queryAppConfigDataWithLogin();
                if (appConfigEntity == null) {
                    LogKit.info("飞书登录集成,报表轻应用跳转,没有获取到登录飞书凭证");
                    filterChain.doFilter(req, res);
                    return;
                }
                String loginContent = "<!DOCTYPE html>\n" +
                        "<html lang=\"en\">\n" +
                        "<head>\n" +
                        "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n" +
                        "    <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\"/>\n" +
                        "    <title>数据决策系统</title>\n" +
                        "    <script type=\"text/javascript\"\n" +
                        "            src=\"/webroot/decision/file?path=/com/fr/plugin/third/party/jsdjjed/sdk/h5-js-sdk-1.5.12.js&type=plain&parser=plain\"></script>\n" +
                        "    <script type=\"text/javascript\"\n" +
                        "            src=\"/webroot/decision/file?path=/com/fr/plugin/third/party/jsdjjed/sdk/runworkhelp.js&type=plain&parser=plain\"></script>\n" +
                        "    <script type=\"text/javascript\"\n" +
                        "            src=\"/webroot/decision/file?path=/com/fr/plugin/third/party/jsdjjed/sdk/vconsole.min.js&type=plain&parser=plain\"></script>\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "<h2>轻应用跳转中,请稍等...</h2>\n" +
                        //"<button onclick=\"test()\">测试</button>"+
                        "<script type=\"text/javascript\">\n" +
                        "    var runWorkH5 = new window.runworkHelp.RunWorkH5({\n" +
                        "        env: \"uat\",\n" +
                        "        devUser: \"xm\",\n" +
                        "        isLocal: false,\n" +
                        "        appId: \"" + appConfigEntity.getAppId() + "\",\n" +
                        "        appSecret: \"" + appConfigEntity.getAppSecret() + "\",\n" +
                        "        isHrInfo: true,\n" +
                        "        isCross: true\n" +
                        "    });\n" +
                        "    runWorkH5.initReady().then(() => {\n" +
                    
                        "        if (runWorkH5.user && runWorkH5.user.user_id) {\n" +
                        "            var state = \"" + state + "\";\n" +
                        "            window.location = \"/webroot/decision?login=feishu&light_app_id=\" + runWorkH5.user.user_id + \"&state=\" + state;\n" +
                        "        }\n" +
                        "else  {\n" +
                        "document.write('<h3>获取user_id出错,请联系管理员</h3>');\n" +
                        "}" +
                        "    });\n" +
                        "function test(){  \n" +
                        "\n" +
                        " var vConsole = new window.VConsole();\n" +
                        " \n" +
                        " vConsole.show();\n" +
                        "console.log('Hello world');\n" +
                        "  debugger;\n" +
                        "  var runWorkH5 = new window.runworkHelp.RunWorkH5({\n" +
                        "        env: \"uat\",\n" +
                        "        devUser: \"xm\",\n" +
                        "        isLocal: false,\n" +
                        "        appId: \"" + appConfigEntity.getAppId() + "\",\n" +
                        "        appSecret: \"" + appConfigEntity.getAppSecret() + "\",\n" +
                        "        isHrInfo: true,\n" +
                        "        isCross: true\n" +
                        "    });\n" +
                        "\t\n" +
                        "vConsole.log(runWorkH5);\t\n" +
                        "    runWorkH5.initReady().then(() => {\n" +
                        "  debugger;\n" +
                        "vConsole.log(runWorkH5);\n" +
                        "if (runWorkH5.user && runWorkH5.user.user_id) {\n" +
                        "document.write('<h3>runWorkH5.user.user_id:'+runWorkH5.user.user_id+'</h3>');\n" +
                        "}\n" +
                        "\n" +
                        "if (runWorkH5.user && runWorkH5.user.name) {\n" +
                        "document.write('<h3>runWorkH5.user.name:'+runWorkH5.user.name+'</h3>');\n" +
                        "}\n" +
                        "\n" +
                        "if (runWorkH5.user && runWorkH5.user.en_name) {\n" +
                        "document.write('<h3>runWorkH5.user.en_name:'+runWorkH5.user.en_name+'</h3>');\n" +
                        "}\n" +
                        "\n" +
                        "if (runWorkH5.user && runWorkH5.user.open_id) {\n" +
                        "document.write('<h3>runWorkH5.user.open_id:'+runWorkH5.user.open_id+'</h3>');\n" +
                        "}    });\n" +
                        "\n" +
                        "}" +
                        "</script>\n" +
                        "</body>\n" +
                        "</html>\n";
                res.setContentType("text/html;charset=UTF-8");
                WebUtils.printAsString(res, loginContent);
                res.setStatus(200);
                return;
            }

            String loginUsername = getOauthLoginUsername(req);
            if (StringKit.isEmpty(loginUsername)) {
                filterChain.doFilter(req, res);
                return;
            }
            LogKit.info("飞书登录集成,用户名:" + loginUsername);

            User user = UserService.getInstance().getUserByUserName(loginUsername);
            boolean tipsOption = false;
            String tipsContent = "";
            if (user == null) {
                tipsOption = true;
                LogKit.info("飞书登录集成,用户名:" + loginUsername + "在报表平台不存在");
                tipsContent = "在报表服务器上不存在";
            } else if (!user.isEnable()) {
                tipsOption = true;
                LogKit.info("飞书登录集成,用户名:" + loginUsername + "在报表平台上被禁用");
                tipsContent = "在报表平台上被禁用";
            }

            //添加认证
            if (!PluginContexts.currentContext().isAvailable()) {
                tipsOption = true;
                LogKit.error("飞书登录集成插件试用过期, 请购买许可证");
                tipsContent = "飞书登录集成插件试用过期, 请购买许可证";
            }

            if (tipsOption) {
                String jumpContent = "<!doctype html>\n" +
                        "<head>\n" +
                        "    <meta charset=\"utf-8\" />\n" +
                        "    <title>提示</title>\n" +
                        "\t\n" +
                        "<script type=\"text/javascript\">\n" +
                        "window.onload=function(){\n" +
                        "            alert(\"用户:" + loginUsername + tipsContent + ",请联系管理员!\");\n" +
                        "            window.location = \"" + AppDataConfig.getInstance().getFrUrl() + "\";\n" +
                        "}\n" +
                        "</script>\t\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "    <div style=\"width: 100%;height:200px;  line-height: 200px;font-size:30px;vertical-align:middle;text-align:center\">\n" +
                        "        <span id=\"show\"></span>\n" +
                        "    </div>\n" +
                        "</body>\n" +
                        "</html>";
                res.setContentType("text/html;charset=UTF-8");
                WebUtils.printAsString(res, jumpContent);
                res.setStatus(200);
                return;
            }

            loginUsername = user.getUserName();
            LogKit.info("飞书登录集成,报表平台用户名:" + loginUsername);


            String loginToken = LoginService.getInstance().login(req, res, loginUsername);
            req.setAttribute("fine_auth_token", loginToken);

            String reqUrl1 = getRealUrl(req);
            if (StringKit.isNotEmpty(reqUrl1) && (!StringKit.equals(reqUrl1, fullUrl))) {
                sendRedirect(res, reqUrl1);
                return;
            }
            filterChain.doFilter(req, res);
        } catch (Exception e) {
            LogKit.error("飞书登录集成出错," + e.getMessage(), e);
        }
    }

    private String getRealUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String state = WebUtils.getHTTPRequestParameter(req, "state");
        if (StringKit.isEmpty(state)) {
            return "";
        }
        String url = AppDataConfig.getInstance().getMapUrl(state);
        url = addLoginPara(url);
        return url;
    }

    private String addLoginPara(String url) {
        if (StringKit.isEmpty(url)) {
            return "";
        }
        if ((url.indexOf("&login=feishu") >= 0) || (url.indexOf("?login=feishu") >= 0)) {
            return url;
        }
        if (url.indexOf("?") >= 0) {
            url = url + "&login=feishu";
            return url;
        }
        url = url + "?login=feishu";
        return url;
    }


    private String getFullAuthorizeUrl(String state) throws Exception {
   
        AppConfigEntity configEntity = AppConfigData.queryAppConfigDataWithLogin();
        if (configEntity == null) {
            LogKit.error("飞书登录集成,没有登录验证的应用凭证信息0");
            return "";
        }
        AppDataConfig config = AppDataConfig.getInstance();
        String tempUrl = config.getFrUrl();
        String url = config.getAuthorizeUrl() + "?app_id=" + configEntity.getAppId() + "&redirect_uri=" + tempUrl + "&state=" + state;
        LogKit.info("飞书登录集成,请求用户授权地址:" + url);
        return url;
    }


    private String getOauthLoginUsername(HttpServletRequest req) throws Exception {
        if (req == null) {
            return "";
        }
        String lightAppId = StringKit.trim(WebUtils.getHTTPRequestParameter(req, "light_app_id"));
        if (StringKit.isNotEmpty(lightAppId)) {
            lightAppId = lightAppId.toUpperCase();
            return lightAppId;
        }

        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "code");
        if (StringKit.isEmpty(oAuthCode)) {
            return "";
        }
        LogKit.info("飞书登录集成,OAuth Code:" + oAuthCode);


        AppConfigEntity configEntity = AppConfigData.queryAppConfigDataWithLogin();
        if (configEntity == null) {
            LogKit.error("飞书登录集成,没有登录验证的应用凭证信息1");
            return "";
        }
        String accessToken = FeishuApp.createAppAccessToken(configEntity.getAppId(), configEntity.getAppSecret());
        if (StringKit.isEmpty(accessToken)) {
            LogKit.error("飞书登录集成,获取app_access_token为空");
            return "";
        }
        FeishuLoginUserBean loginUserBean = FeishuApp.getLoginUserInfo(oAuthCode, accessToken);
        if (loginUserBean == null) {
            LogKit.error("飞书登录集成,获取登录信息为空");
            return "";
        }

        String loginMap = configEntity.getLoginMap();
        QueryCondition queryCondition = getUserQueryCondition(loginMap, loginUserBean);
        if (queryCondition == null) {
            LogKit.error("飞书登录集成,用户信息没有有效的映射匹配");
            return "";
        }
        List<User> perfectMatchUsers = AuthorityContext.getInstance().getUserController().find(queryCondition);
        if ((perfectMatchUsers == null) || (perfectMatchUsers.size() <= 0)) {
            LogKit.error("飞书登录集成,没有匹配到用户信息");
            return "";
        }
        if (perfectMatchUsers.size() >= 2) {
            LogKit.error("飞书登录集成,匹配到多个用户信息");
            return "";
        }
        User user = perfectMatchUsers.get(0);
        String username = user.getUserName();
        return username;
    }


    private QueryCondition getUserQueryCondition(String loginMap, FeishuLoginUserBean loginUserBean) {
        if (StringKit.isEmpty(loginMap)) {
            loginMap = "name";
        }
        Set<String> usernameValues = new HashSet<>();
        String[] loginNames = loginMap.split(",");
        String loginName;
        String mobile = "";
        for (int i = 0, max = loginNames.length - 1; i <= max; i++) {
            loginName = StringKit.trim(loginNames[i]);
            if (StringKit.isEmpty(loginName)) {
                continue;
            }
            if (StringKit.equalsIgnoreCase("name", loginName) && StringKit.isNotEmpty(loginUserBean.getName())) {
                usernameValues.add(loginUserBean.getName());
            } else if (StringKit.equalsIgnoreCase("en_name", loginName) && StringKit.isNotEmpty(loginUserBean.getEnName())) {
                usernameValues.add(loginUserBean.getEnName());
            } else if (StringKit.equalsIgnoreCase("user_id", loginName) && StringKit.isNotEmpty(loginUserBean.getUserId())) {
                usernameValues.add(loginUserBean.getUserId());
                usernameValues.add(loginUserBean.getUserId().toUpperCase());
            } else if (StringKit.equalsIgnoreCase("mobile", loginName) && StringKit.isNotEmpty(loginUserBean.getMobile())) {
                mobile = loginUserBean.getMobile();
            }
        }

        int queryCount = 0;
        QueryCondition queryCondition = QueryConditionKit.newQueryCondition();
        if (usernameValues.size() >= 1) {
            queryCount++;
            queryCondition.addRestriction(RestrictionKit.in("userName", usernameValues));
        }
        if (StringKit.isNotEmpty(mobile)) {
            queryCount++;
            queryCondition.addRestriction(RestrictionKit.eq("mobile", mobile));
        }
        if (queryCount <= 0) {
            return null;
        }
        return queryCondition;
    }


    public synchronized static String getSysTime() {
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date();
        String nowData = format.format(date);
        return nowData;
    }

    private void sendRedirect(HttpServletResponse res, String url) {
        res.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }

    private String getRequestUrl(HttpServletRequest req) {
        String fullUrl = req.getRequestURL().toString();
        fullUrl = getRealUrl(fullUrl);
        Map<String, String[]> paraMap = req.getParameterMap();
        String paraName;
        String[] paraValues;
        String loginTypeParaName = AppDataConfig.getInstance().getLoginTypeNameParameter();
        String queryStr = "";
        for (Map.Entry<String, String[]> entry : paraMap.entrySet()) {
            paraName = entry.getKey();
            if (ComparatorUtils.equals(paraName, loginTypeParaName)) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "code")) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "state")) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "login_type")) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "light_app")) {
                continue;
            }
            paraValues = entry.getValue();
            queryStr = addParaToQuery(queryStr, paraName, paraValues);
        }
        if (StringKit.isEmpty(queryStr)) {
            return fullUrl;
        }
        fullUrl = fullUrl + "?" + queryStr;
        return fullUrl;
    }

    private String getRealUrl(String url) {
        if (StringKit.isEmpty(url)) {
            return url;
        }
        int index = url.indexOf("/decision");
        if (index < 0) {
            return url;
        }
        String tempUrl = AppDataConfig.getInstance().getFrUrl() + url.substring(index + "/decision".length());
        return tempUrl;
    }


    private String addParaToQuery(String query, String paraName, String[] paraValues) {
        if (StringKit.isEmpty(paraName)) {
            return query;
        }
        String fullQuery = query;
        if ((paraValues == null) || (paraValues.length <= 0)) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = fullQuery + paraName + "=";
            return fullQuery;
        }
        for (int i = 0, max = paraValues.length - 1; i <= max; i++) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = fullQuery + paraName + "=" + Utils.encodeUrlWithUtf8(paraValues[i]);
        }
        return fullQuery;
    }


    private boolean isAllowLoginWithParameter(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        String loginTypeNameParameter = "login_type";
        String loginTypeConfigValue = "feishu";
        if (StringKit.isEmpty(loginTypeNameParameter) || StringKit.isEmpty(loginTypeConfigValue)) {
            return false;
        }
        String loginTypeValue = WebUtils.getHTTPRequestParameter(req, loginTypeNameParameter);
        return ComparatorUtils.equalsIgnoreCase(loginTypeConfigValue, loginTypeValue);
    }


    private boolean isAllowLoginWithLightApp(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        String loginTypeNameParameter = "light_app";
        String loginTypeConfigValue = "feishu";
        if (StringKit.isEmpty(loginTypeNameParameter) || StringKit.isEmpty(loginTypeConfigValue)) {
            return false;
        }
        String loginTypeValue = WebUtils.getHTTPRequestParameter(req, loginTypeNameParameter);
        return ComparatorUtils.equalsIgnoreCase(loginTypeConfigValue, loginTypeValue);
    }


    private boolean isReportRequest(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String url = req.getRequestURL().toString();
        if (url.endsWith("/decision") || url.endsWith("/decision/")) {
            return true;
        }
        if ((url.indexOf("/decision/") >= 0) && (url.indexOf("/entry/access/") >= 0)) {
            return true;
        }
        String viewlet = WebUtils.getHTTPRequestParameter(req, "viewlet");
        if ((url.indexOf("/decision/view/report") >= 0) && (StringKit.isNotEmpty(viewlet))) {
            return true;
        }
        if ((url.indexOf("/decision/view/form") >= 0) && (StringKit.isNotEmpty(viewlet))) {
            return true;
        }
        return false;
    }

    private boolean isOauthCodeRequest(HttpServletRequest req) throws IOException {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "code");
        if (StringKit.isNotEmpty(oAuthCode)) {
            return true;
        }
        return false;
    }


    private boolean isNoAuthRequest(HttpServletRequest req) throws IOException {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "loginType");
        if (StringKit.equalsIgnoreCase("noauth", oAuthCode)) {
            return true;
        }
        return false;
    }

}
