package com.fr.plugin.third.party.jsdjjed.schedule;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.DataSetFunctionParameterMapNameSpace;
import com.fr.base.TemplateUtils;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.User;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.third.party.jsdjjed.app.config.AppConfigData;
import com.fr.plugin.third.party.jsdjjed.app.config.AppConfigEntity;
import com.fr.plugin.third.party.jsdjjed.feishu.FeishuApp;
import com.fr.plugin.third.party.jsdjjed.schedule.bean.OutputAppMessagePush;
import com.fr.schedule.base.constant.ScheduleConstants;
import com.fr.schedule.extension.report.job.output.BaseOutputFormat;
import com.fr.schedule.feature.output.OutputActionHandler;
import com.fr.script.Calculator;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.third.org.apache.commons.collections4.CollectionUtils;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;

public class AppMessagePushHandler extends OutputActionHandler<OutputAppMessagePush> {


    @Override
    public void doAction(OutputAppMessagePush action, Map<String, Object> map) throws Exception {
        LogKit.info("飞书发送消息,开发发送任务........");
        String configId = action.getConfigId();
        if (StringKit.isEmpty(configId)) {
            return;
        }

        //添加认证
        if (!PluginContexts.currentContext().isAvailable()) {
            LogKit.error("飞书登录集成插件试用过期,飞书发送消息, 请购买许可证");
        }


        AppConfigEntity configEntity = AppConfigData.queryAppConfigDataWithConfigIdAndOnlyOne(configId);
        if (configEntity == null) {
            return;
        }

        LogKit.info("飞书发送消息,获取token........");
        String appAccessToken = FeishuApp.createAppAccessToken(configEntity.getAppId(), configEntity.getAppSecret());
        if (StringKit.isEmpty(appAccessToken)) {
            LogKit.error("飞书发送消息,获取app_access_token为空");
            return;
        }

        LogKit.info("飞书发送消息,获取飞书用户和群........");
        List<String> feishuUserIds = getFeishuUserIds(map);
        String chatGroupId = action.getChatGroupId();
        if ("empty_group".equalsIgnoreCase(chatGroupId)) {
            chatGroupId = "";
        }

        if (CollectionUtils.isEmpty(feishuUserIds) && StringKit.isEmpty(chatGroupId)) {
            LogKit.error("飞书发送消息,飞书用户ID和群ID都为空");
            return;
        }

        String msgType = FeishuApp.getMsgType(action.getFeishuMsgType());
        String sendType = FeishuApp.getSendType(action.getFeishuSendType());
        LogKit.info("飞书发送消息,消息类型:" + FeishuApp.getMsgTypeNotes(msgType));
        LogKit.info("飞书发送消息,发送类型:" + FeishuApp.getSendTypeNotes(sendType));
        if (FeishuApp.isSendMessage(sendType)) {
            LogKit.info("飞书发送消息,生成发送文本........");
            Calculator cal = Calculator.createCalculator();
            String content = action.getContent();
            String msgContent = TemplateUtils.renderTpl(cal, content);
            LogKit.info("飞书发送消息,发送文本:" + msgContent);

            LogKit.info("飞书发送消息,发送文本消息........");
            if (StringKit.isNotEmpty(chatGroupId) && StringKit.isNotEmpty(msgContent)) {
                LogKit.info("飞书发送消息,发送群文本消息........");
                if ("text".equals(msgType)) {
                    FeishuApp.sendGroupMessageWithText(chatGroupId, msgContent, appAccessToken);
                } else {
                    FeishuApp.sendMessage("chat_id", chatGroupId, msgContent, "post", appAccessToken);
                }
            }

            if (CollectionUtils.isNotEmpty(feishuUserIds) && StringKit.isNotEmpty(msgContent)) {
                LogKit.info("飞书发送消息,发送个人文本消息........");
                if ("text".equals(msgType)) {
                    FeishuApp.sendBatchMessageWithText(feishuUserIds, msgContent, appAccessToken);
                } else {
                    FeishuApp.sendBatchMessageWithRichText(feishuUserIds, msgContent, appAccessToken);
                }
            }
        }

        if (FeishuApp.isSendFile(sendType)) {
            LogKit.info("飞书发送消息,发送文件消息........");
            JSONObject commonUploadJson = getUploadFileJson(action, map, appAccessToken);
            if (FeishuApp.isValidFileJson(commonUploadJson)) {
                if (StringKit.isNotEmpty(chatGroupId)) {
                    LogKit.info("飞书发送消息,发送群文件消息........");
                    FeishuApp.sendGroupMessageWithFile(chatGroupId, commonUploadJson.toString(), appAccessToken);
                }
            } else if (FeishuApp.isValidImageJson(commonUploadJson)) {
                if (StringKit.isNotEmpty(chatGroupId)) {
                    LogKit.info("飞书发送消息,发送群图片消息........");
                    FeishuApp.sendGroupMessageWithImage(chatGroupId, commonUploadJson.toString(), appAccessToken);
                }
            }

            if (CollectionUtils.isNotEmpty(feishuUserIds)) {
                List<String> userIds = getTaskUserIds(map);
                String feishuUserId, userId;
                List<String> sendUserIds = new ArrayList<>();
                JSONObject uploadJson;
                List<String> tempFeishuUserIds = new ArrayList<>();
                for (int i = 0, max = userIds.size() - 1; i <= max; i++) {
                    userId = userIds.get(i);
                    feishuUserId = toLower(userId);
                    if (sendUserIds.contains(feishuUserId)) {
                        continue;
                    }
                    tempFeishuUserIds.clear();
                    tempFeishuUserIds.add(feishuUserId);
                    uploadJson = getUserUploadFileJson(action, map, appAccessToken, userId);
                    if ((!FeishuApp.isValidFileJson(uploadJson)) && (!FeishuApp.isValidImageJson(uploadJson))) {
                        uploadJson = commonUploadJson;
                    }
                    if (FeishuApp.isValidFileJson(uploadJson)) {
                        if (CollectionUtils.isNotEmpty(tempFeishuUserIds)) {
                            LogKit.info("飞书发送消息,发送个人文件消息........");
                            FeishuApp.sendBatchMessageWithFile(tempFeishuUserIds, uploadJson.toString(), appAccessToken);
                        }
                    } else if (FeishuApp.isValidImageJson(uploadJson)) {
                        if (CollectionUtils.isNotEmpty(tempFeishuUserIds)) {
                            LogKit.info("飞书发送消息,发送个人图片消息........");
                            FeishuApp.sendBatchMessageWithImage(tempFeishuUserIds, uploadJson.toString(), appAccessToken);
                        }
                    }
                    sendUserIds.add(feishuUserId);
                }
            }
        }
        LogKit.info("飞书发送消息,结束发送任务........");
    }


    private JSONObject getUserUploadFileJson(OutputAppMessagePush action, Map<String, Object> map, String appAccessToken, String userId) throws IOException {
        JSONObject fileJson = new JSONObject();
        String fileType = getFileType(action);
        if (StringKit.isEmpty(fileType)) {
            return fileJson;
        }

        String filePath = getUploadFilePath(action, map);
        String fileName = getUploadFileName(filePath);

        String userFilePath = getUserUploadFilePath(action, map, userId, fileName);
        if (StringKit.isEmpty(userFilePath)) {
            return fileJson;
        }

        InputStream fileInputStream = getUploadFileInputStream(userFilePath);
        if (fileInputStream == null) {
            return fileJson;
        }

        if ("png".equals(fileType)) {
            fileJson = FeishuApp.uploadImage(fileInputStream, appAccessToken);
            return fileJson;
        }

        fileJson = FeishuApp.uploadFile(fileType, fileName, fileInputStream, appAccessToken);
        return fileJson;
    }

    private String getUserUploadFilePath(OutputAppMessagePush action, Map<String, Object> map, String userId, String fileName) {
        String fileType = getFileType(action);
        if (StringKit.isEmpty(fileType)) {
            return "";
        }

        String fileSuffix = "." + fileType;
        if (StringKit.isEmpty(fileSuffix)) {
            return "";
        }
        String tempFilePath = String.valueOf(map.get(ScheduleConstants.SAVE_DIRECTORY_WITHOUT_USERNAME));
        if (StringKit.isEmpty(tempFilePath)) {
            return "";
        }
        String filePath = tempFilePath.substring(0, tempFilePath.lastIndexOf("/")) + "/" + userId + "/" + fileName;
        if (StringKit.isNotEmpty(filePath) && ResourceIOUtils.exist(filePath)) {
            return filePath;
        }
        return "";
    }

    private JSONObject getUploadFileJson(OutputAppMessagePush action, Map<String, Object> map, String appAccessToken) throws IOException {
        JSONObject fileJson = new JSONObject();
        String fileType = getFileType(action);
        if (StringKit.isEmpty(fileType)) {
            return fileJson;
        }

        String filePath = getUploadFilePath(action, map);
        InputStream fileInputStream = getUploadFileInputStream(filePath);
        if (fileInputStream == null) {
            return fileJson;
        }

        if ("png".equals(fileType)) {
            fileJson = FeishuApp.uploadImage(fileInputStream, appAccessToken);
            return fileJson;
        }

        String fileName = getUploadFileName(filePath);
        fileJson = FeishuApp.uploadFile(fileType, fileName, fileInputStream, appAccessToken);
        return fileJson;
    }


    private String getFileType(OutputAppMessagePush action) {
        if (action == null) {
            return "";
        }

        if (action.getType() != 3) {
            return "";
        }
        BaseOutputFormat outputFormat = BaseOutputFormat.fromInteger(action.getMediaId());
        if (outputFormat == null) {
            return "";
        }
        String fileSuffix = StringKit.trim(outputFormat.getFileSuffix());
        if (StringKit.isEmpty(fileSuffix)) {
            return "";
        }
        String type = "";
        if (fileSuffix.startsWith(".")) {
            type = fileSuffix.substring(1);
        }
        type = StringKit.trim(type);
        return type;
    }

    private String getUploadFilePath(OutputAppMessagePush action, Map<String, Object> map) {
        String fileType = getFileType(action);
        if (StringKit.isEmpty(fileType)) {
            return "";
        }

        String fileSuffix = "." + fileType;
        if (StringKit.isEmpty(fileSuffix)) {
            return "";
        }
        String[] filePaths = (String[]) map.get(ScheduleConstants.OUTPUT_FILES);
        if ((filePaths == null) || (filePaths.length <= 0)) {
            return "";
        }
        String filePath;
        for (int i = 0, max = filePaths.length - 1; i <= max; i++) {
            filePath = filePaths[i];
            if (StringKit.isNotEmpty(filePath) && filePath.endsWith(fileSuffix) && ResourceIOUtils.exist(filePath)) {
                return filePath;
            }
        }
        return "";
    }

    private String getUploadFileName(String filePath) {
        if (StringKit.isEmpty(filePath)) {
            return "";
        }
        String name = ResourceIOUtils.getName(filePath);
        return name;
    }

    private InputStream getUploadFileInputStream(String filePath) {
        if (StringKit.isEmpty(filePath)) {
            return null;
        }
        if (ResourceIOUtils.exist(filePath)) {
            InputStream in = ResourceIOUtils.read(filePath);
            return in;
        }
        return null;
    }


    private List<String> getTaskUserIds(Map<String, Object> map) throws Exception {
        if ((map == null) || (map.size() <= 0)) {
            return new ArrayList<>();
        }
        String[] scheduleUserNames = (String[]) map.get(ScheduleConstants.USERNAMES);
        String scheduleUserName = (String) map.get(ScheduleConstants.USERNAME);
        Set<String> userSet = new HashSet();
        String tempUserName = StringKit.trim(scheduleUserName);
        if (StringKit.isNotEmpty(tempUserName)) {
            userSet.add(tempUserName);
        }

        if ((scheduleUserNames != null) && (scheduleUserNames.length >= 1)) {
            for (int i = 0, max = scheduleUserNames.length - 1; i <= max; i++) {
                tempUserName = StringKit.trim(scheduleUserNames[i]);
                if (StringKit.isNotEmpty(tempUserName)) {
                    userSet.add(tempUserName);
                }
            }
        }
        if (userSet.size() <= 0) {
            return new ArrayList<>();
        }
        List<String> userId = new ArrayList<>();
        userId.addAll(userSet);
        return userId;
    }

    private List<String> getFeishuUserIds(Map<String, Object> map) throws Exception {
        if ((map == null) || (map.size() <= 0)) {
            return new ArrayList<>();
        }
        String[] scheduleUserNames = (String[]) map.get(ScheduleConstants.USERNAMES);
        String scheduleUserName = (String) map.get(ScheduleConstants.USERNAME);
        Set<String> userSet = new HashSet();
        String tempUserName = toLower(scheduleUserName);
        if (StringKit.isNotEmpty(tempUserName)) {
            userSet.add(tempUserName);
        }

        if ((scheduleUserNames != null) && (scheduleUserNames.length >= 1)) {
            for (int i = 0, max = scheduleUserNames.length - 1; i <= max; i++) {
                tempUserName = toLower(scheduleUserNames[i]);
                if (StringKit.isNotEmpty(tempUserName)) {
                    userSet.add(tempUserName);
                }
            }
        }
        if (userSet.size() <= 0) {
            return new ArrayList<>();
        }
        List<String> userId = new ArrayList<>();
        userId.addAll(userSet);
        return userId;
    }

    private String toLower(String value) {
        if (StringKit.isEmpty(value)) {
            return "";
        }
        String tempValue = StringKit.trim(value.toLowerCase());
        if (StringKit.isEmpty(tempValue)) {
            return "";
        }
        return tempValue;
    }


    private List<String> getFeishuUserId(String appAccessToken, Map<String, Object> map) throws Exception {
        if (StringKit.isEmpty(appAccessToken)) {
            return new ArrayList<>();
        }

        if ((map == null) || (map.size() <= 0)) {
            return new ArrayList<>();
        }
        String[] scheduleUserNames = (String[]) map.get(ScheduleConstants.USERNAMES);
        String scheduleUserName = (String) map.get(ScheduleConstants.USERNAME);
        Set<String> userSet = new HashSet();
        if (StringUtils.isNotBlank(scheduleUserName)) {
            userSet.add(scheduleUserName);
        }
        if ((scheduleUserNames != null) && (scheduleUserNames.length >= 1)) {
            userSet.addAll(Arrays.asList(scheduleUserNames));
        }

        List<User> userList = AuthorityContext.getInstance().getUserController().find(QueryFactory.create().addRestriction(RestrictionFactory.in("userName", userSet)));
        if ((userList == null) || (userList.size() <= 0)) {
            return new ArrayList<>();
        }

        Set<String> emailSet = new HashSet();
        Set<String> mobileSet = new HashSet();

        User user;
        String email, mobile;
        for (int i = 0, max = userList.size() - 1; i <= max; i++) {
            user = userList.get(i);
            email = StringKit.trim(user.getEmail());
            if (StringKit.isNotEmpty(email)) {
                emailSet.add(email);
            }
            mobile = StringKit.trim(user.getMobile());
            if (StringKit.isNotEmpty(mobile)) {
                mobileSet.add(mobile);
            }
        }

        if ((emailSet.size() <= 0) && (mobileSet.size() <= 0)) {
            return new ArrayList<>();
        }

        List<String> feishuUserIds = FeishuApp.getFeishuUserId(emailSet, mobileSet, appAccessToken);
        return feishuUserIds;
    }
}
